SET SESSION FOREIGN_KEY_CHECKS=0;

/* Drop Tables */

DROP TABLE IF EXISTS attnd_holiday;
DROP TABLE IF EXISTS attnd_violation;
DROP TABLE IF EXISTS attnd_worktime;
DROP TABLE IF EXISTS deduct;
DROP TABLE IF EXISTS deduct_social_insurance;
DROP TABLE IF EXISTS deduct_tax;
DROP TABLE IF EXISTS paid_salary;
DROP TABLE IF EXISTS paid_basic;
DROP TABLE IF EXISTS pay_statement;
DROP TABLE IF EXISTS ps_design_info;




/* Create Tables */

-- 勤務-休暇
CREATE TABLE attnd_holiday
(
	pay_date date NOT NULL COMMENT '支給日',
	special_days int COMMENT '特休日数',
	paid_used_days int COMMENT '有休使用日数',
	paid_remain_of_last_year int COMMENT '有休残前年',
	paid_remain int COMMENT '有休残本数',
	PRIMARY KEY (pay_date)
) COMMENT = '勤務-休暇';


-- 勤務-規則違反
CREATE TABLE attnd_violation
(
	pay_date date NOT NULL COMMENT '支給日',
	late_early_count int COMMENT '遅早回数',
	late_early_time decimal(5,2) COMMENT '遅早時間',
	deduct_time decimal(5,2) COMMENT '控除時間',
	absent_days int COMMENT '欠勤日数',
	PRIMARY KEY (pay_date)
) COMMENT = '勤務-規則違反';


-- 勤怠-労働時間
CREATE TABLE attnd_worktime
(
	pay_date date NOT NULL COMMENT '支給日',
	official_time decimal(5,2) NOT NULL COMMENT '本社所定時間',
	over_time decimal(5,2) COMMENT '残業時間',
	midnight_time decimal(5,2) COMMENT '深夜勤務時間',
	legal_holiday_time decimal(5,2) COMMENT '法定休日時間',
	PRIMARY KEY (pay_date)
) COMMENT = '勤怠-労働時間';


-- 控除
CREATE TABLE deduct
(
	pay_date date NOT NULL COMMENT '支給日',
	social_insurance int COMMENT '社会保険料',
	tax int COMMENT '税金',
	dues int COMMENT '会費',
	exam_fee int COMMENT '受験料',
	year_end_adjust_result int COMMENT '年調結果',
	PRIMARY KEY (pay_date)
) COMMENT = '控除';


-- 控除-社会保険料
CREATE TABLE deduct_social_insurance
(
	pay_date date NOT NULL COMMENT '支給日',
	health_insurance int COMMENT '健康保険',
	nursing_insurance int COMMENT '介護保険',
	employee_pension int COMMENT '厚生年金',
	employment_insurance int COMMENT '雇用保険',
	harmonization int COMMENT '社会保険調整',
	PRIMARY KEY (pay_date)
) COMMENT = '控除-社会保険料';


-- 控除-税金
CREATE TABLE deduct_tax
(
	pay_date date NOT NULL COMMENT '支給日',
	income_tax int COMMENT '所得税',
	inhabitant_tax int COMMENT '住民税',
	net_taxable_amount int COMMENT '課税対象額',
	PRIMARY KEY (pay_date)
) COMMENT = '控除-税金';


-- 基本支給
CREATE TABLE paid_basic
(
	basic_id int NOT NULL COMMENT '基本支給ID',
	revision_date date COMMENT '改定日',
	base_salary int COMMENT '基本給',
	executive_allowance int COMMENT '役職手当',
	technology_allowance int COMMENT '技術手当',
	business_allowance int COMMENT '業務手当',
	capacity_allowance int COMMENT '資格手当',
	housing_allowance int COMMENT '住宅手当',
	family_allowance int COMMENT '家族手当',
	sales_allowance int COMMENT '営業手当',
	PRIMARY KEY (basic_id)
) COMMENT = '基本支給';


-- 支給額
CREATE TABLE paid_salary
(
	pay_date date NOT NULL COMMENT '支給日',
	basic_id int NOT NULL COMMENT '基本支給ID',
	legal_holiday_allowance int COMMENT '法定休日手当',
	commute_allowance int COMMENT '通勤手当',
	commute_allowance_diff int COMMENT '通勤手当差額',
	special_allowance int COMMENT '特別手当',
	over_allowance int COMMENT '時間外手当',
	midnight_allowance int COMMENT '深夜勤務手当',
	violation_deduct int COMMENT '遅早欠勤控除',
	retroactivity_paid int COMMENT '遡及支給',
	bonus int DEFAULT 0 COMMENT '賞与',
	PRIMARY KEY (pay_date)
) COMMENT = '支給額';


-- 給与明細
CREATE TABLE pay_statement
(
	pay_date date NOT NULL COMMENT '支給日',
	total_amount_paid int COMMENT '総支給額',
	deduct_total int COMMENT '控除合計',
	net_amount_paid int COMMENT '差引支給額',
	bonus_flag int DEFAULT 0 COMMENT '賞与フラグ',
	PRIMARY KEY (pay_date)
) COMMENT = '給与明細';


-- 給与支給デザイン情報
CREATE TABLE ps_design_info
(
	id int NOT NULL COMMENT 'id',
	category varchar(20) COMMENT 'カテゴリ',
	caption_eng varchar(30) COMMENT 'キャプション(英語)',
	caption_jp varchar(30) COMMENT 'キャプション(日本語)',
	request_param_name varchar(30) COMMENT 'リクエストパラメータ名',
	is_disabled varchar(10) COMMENT '操作無効',
	pay_enabled varchar(10) COMMENT '給与有効',
	bonus_enabled varchar(10) COMMENT '賞与有効',
	db_table_name varchar(30) COMMENT 'DBテーブル名',
	db_col_name varchar(30) COMMENT 'DB列名',
	class varchar(20) COMMENT 'クラス名',
	comment varchar(100) COMMENT 'コメント',
	PRIMARY KEY (id)
) COMMENT = '給与支給デザイン情報';



