package controllers.ext.sal;

import java.io.IOException;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import models.ext.dba.sal.DBAPayStatementControlBean;
import models.ext.sal.SALAttendValueBean;
import models.ext.sal.SALBasicData;
import models.ext.sal.SALDeductSocialInsuranceValueBean;
import models.ext.sal.SALDeductTaxValueBean;
import models.ext.sal.SALDeductValueBean;
import models.ext.sal.SALPaidBasicValueBean;
import models.ext.sal.SALPaidValueBean;
import models.ext.sal.SALPayHistoryValueBean;
import models.ext.sal.SALPayStatementValueBean;
import models.ueat.dba.DBAControlBeanInterface;
import models.ueat.exp.EXPConditionValueBean;
import models.ueat.exp.EXPHighChartValueBean;
import util.com.COMDataDump;
import util.com.COMDef;
import util.ext.sal.SALDef;
import controllers.ueat.com.COMRequestManager;
import controllers.ueat.exp.EXPCommonManager;
import controllers.ueat.exp.EXPManagerInterface;


public class SALPayStatementManager
    extends EXPCommonManager implements EXPManagerInterface {

    @Override
    public boolean execute() {
        String requestName = getRequest().getParameter("request");
        String queryPattern = COMRequestManager.getInstance().getQueryPattern(requestName);
        switch (queryPattern) {
        case COMDef.QUERY_PTN_SELECT:
            if (!draw()) {
                return false;
            }
            break;
        case COMDef.QUERY_PTN_INSERT:
            if (!addData()) {
                return false;
            }
            break;
        case COMDef.QUERY_PTN_UPDATE:
            break;
        case COMDef.QUERY_PTN_DELETE:
            break;
        case COMDef.QUERY_PTN_NONE:
            break;
        default:
            break;
        }

        return true;
    }

    private boolean draw() {
        DBAControlBeanInterface control;
        try {
            control = DBAPayStatementControlBean.class.newInstance();

            EXPConditionValueBean condValue = new EXPConditionValueBean();
            String salKind = getRequest().getParameter("SalKind");
            if (salKind.equals(SALDef.SalKind.Bonus.toString())) {
                condValue.setWhereCond(" bonus_flag = 1 ");
            } else {
                condValue.setWhereCond(" bonus_flag = 0 ");
            }
            control.setCondParam(condValue);
            if (!control.select()) {
                return false;
            }

            SALStructHighChartData chData = new SALStructHighChartData();
            EXPHighChartValueBean hcValue = new EXPHighChartValueBean();
            hcValue.setDispHeight(600);
            hcValue.setDispWidth(1000);
            chData.setHighChartValue(hcValue);
            chData.setTargetValue((SALPayHistoryValueBean)control.getResultValue());
            String strChartData = chData.createPayChartData();
            ((SALPayHistoryValueBean)control.getResultValue()).setHighChartData(strChartData);

            // --- Test ---------------------------------
            List packageList = new ArrayList<String>();
            packageList.add("controllers.ueat.exp");
            packageList.add("controllers.ext.sal");
            packageList.add("junit.ueat");
            packageList.add("models.ueat.dba");
            packageList.add("models.ueat.exp");
            packageList.add("models.ueat.com");
            packageList.add("models.ext.dba");
            packageList.add("models.ext.sal");
            COMDataDump dump = new COMDataDump();
            dump.setTargetCategoryFromPackage(packageList);
            SALPayHistoryValueBean phValue = (SALPayHistoryValueBean)control.getResultValue();
            Field field = SALPayHistoryValueBean.class.getDeclaredField("PayStatementList");
            dump.dumpData(phValue, field);
            // --- Test ---------------------------------
            setValueBean(control.getResultValue());
        }
        catch (InstantiationException e) {
            e.printStackTrace();
            return false;
        }
        catch (IllegalAccessException e) {
            e.printStackTrace();
            return false;
        }
        catch (ClassNotFoundException e) {
            e.printStackTrace();
            return false;
        }
        catch (IOException e) {
            e.printStackTrace();
            return false;
        }
        catch (NoSuchFieldException | SecurityException e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

    private boolean addData() {
        DBAControlBeanInterface control;
        try {
            SALBasicData basic = SALBasicData.getInstance();

            control = DBAPayStatementControlBean.class.newInstance();
            SALPayHistoryValueBean value = new SALPayHistoryValueBean();

            SALPayStatementValueBean psValue = new SALPayStatementValueBean();
            psValue.setPayDate(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_PAY_DATE).getReqPrmName()));
            psValue.setTotalAmountPaid(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_TOTAL_AMOUNT_PAID).getReqPrmName())));
            psValue.setDeductTotal(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_DEDUCT_TOTAL).getReqPrmName())));
            psValue.setNetAmountPaid(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_NET_AMOUNT_PAID).getReqPrmName())));
            psValue.setBonusMode(false);
            int  bonusFlag = Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_BONUS_FLAG).getReqPrmName()));
            if (bonusFlag == 1) {
                psValue.setBonusMode(true);
            }

            if (!psValue.isBonusMode()) {
                SALAttendValueBean attendValue = new SALAttendValueBean();
                attendValue.setOfficialTime(Double.parseDouble(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_OFFICIAL_TIME).getReqPrmName())));
                attendValue.setOverTime(Double.parseDouble(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_OVER_TIME).getReqPrmName())));
                attendValue.setMidnightTime(Double.parseDouble(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_MIDNIGHT_TIME).getReqPrmName())));
                attendValue.setLegalHolidayTime(Double.parseDouble(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_LEGAL_HOLIDAY_TIME).getReqPrmName())));
                attendValue.setLateEarlyCount(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_LATE_EARLY_COUNT).getReqPrmName())));
                attendValue.setLateEarlyTime(Double.parseDouble(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_LATE_EARLY_TIME).getReqPrmName())));
                attendValue.setDeductTime(Double.parseDouble(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_DEDUCT_TIME).getReqPrmName())));
                attendValue.setAbsentDays(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_ABSENT_DAYS).getReqPrmName())));
                attendValue.setSpecialDays(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_SPECIAL_DAYS).getReqPrmName())));
                attendValue.setPaidUsedDays(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_PAID_USED_DAYS).getReqPrmName())));
                attendValue.setPaidRemainOfLastYear(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_PAID_REMAIN_OF_LAST_YEAR).getReqPrmName())));
                attendValue.setPaidRemain(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_PAID_REMAIN).getReqPrmName())));
                psValue.setAttendValue(attendValue);
            }

            SALPaidBasicValueBean paidVBasicValue = new SALPaidBasicValueBean();
            if (psValue.isBonusMode()) {
                paidVBasicValue.setBasicId(basic.getDefaultPaidBasicValue().getBasicId());
            } else {
                paidVBasicValue.setBasicId(basic.getNewestPaidBasicValue().getBasicId());
            }
            SALPaidValueBean paidValue = new SALPaidValueBean();
            paidValue.setPaidBasic(paidVBasicValue);
            paidValue.setLegalHolidayAllowance(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_LEGAL_HOLIDAY_ALLOWANCE).getReqPrmName())));
            paidValue.setCommuteAllowance(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_COMMUTE_ALLOWANCE).getReqPrmName())));
            paidValue.setCommuteAllowanceDiff(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_COMMUTE_ALLOWANCE_DIFF).getReqPrmName())));
            paidValue.setSpecialAllowance(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_SPECIAL_ALLOWANCE).getReqPrmName())));
            paidValue.setOverAllowance(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_OVER_ALLOWANCE).getReqPrmName())));
            paidValue.setMidnightAllowance(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_MIDNIGHT_ALLOWANCE).getReqPrmName())));
            paidValue.setViolationDeduct(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_VIOLATION_DEDUCT).getReqPrmName())));
            paidValue.setRetroactivityPaid(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_RETROACTIVITY_PAID).getReqPrmName())));
            paidValue.setBonus(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_BONUS).getReqPrmName())));
            psValue.setPaidValue(paidValue);

            SALDeductSocialInsuranceValueBean deductSocialInsuranceValue = new SALDeductSocialInsuranceValueBean();
            deductSocialInsuranceValue.setHealthInsurance(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_HEALTH_INSURANCE).getReqPrmName())));
            deductSocialInsuranceValue.setNursingInsurance(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_NURSING_INSURANCE).getReqPrmName())));
            deductSocialInsuranceValue.setEmployeePension(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_EMPLOYEE_PENSION).getReqPrmName())));
            deductSocialInsuranceValue.setEmploymentInsurance(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_EMPLOYMENT_INSURANCE).getReqPrmName())));
            deductSocialInsuranceValue.setHarmonization(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_HARMONIZATION).getReqPrmName())));
            SALDeductTaxValueBean deductTaxValue = new SALDeductTaxValueBean();
            deductTaxValue.setIncomeTax(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_INCOME_TAX).getReqPrmName())));
            deductTaxValue.setInhabitantTax(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_INHABITANT_TAX).getReqPrmName())));
            deductTaxValue.setNetTaxableAmount(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_NET_TAXABLE_AMOUNT).getReqPrmName())));
            SALDeductValueBean deductValue = new SALDeductValueBean();
            deductValue.setSocialInsurance(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_SOCIAL_INSURANCE).getReqPrmName())));
            deductValue.setTax(deductTaxValue.getIncomeTax() + deductTaxValue.getInhabitantTax());
            deductValue.setDues(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_DUES).getReqPrmName())));
            deductValue.setExamFee(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_EXAM_FEE).getReqPrmName())));
            deductValue.setYearEndAdjustResult(Integer.parseInt(getRequest().getParameter(basic.getPsDesignElement(SALDef.DB_COL_YEAR_END_ADJUST_RESULT).getReqPrmName())));
            deductValue.setSocialInsuranceValue(deductSocialInsuranceValue);
            deductValue.setTaxValue(deductTaxValue);
            psValue.setDeductValue(deductValue);

            value.getPayStatementList().add(psValue);
            control.setTargetValue(value);

            if (!control.insert()) {
                return false;
            }
        }
        catch (InstantiationException e) {
            e.printStackTrace();
            return false;
        }
        catch (IllegalAccessException e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }
}
