package controllers.ueat.exp;

import java.io.IOException;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import models.ext.sal.SALBasicData;
import models.ueat.exp.EXPCommonValueBean;
import util.com.COMUtility;
import controllers.ueat.com.COMRequestManager;

/**
 * Servlet implementation class EXPMainServlet
 */
public class EXPMainServlet extends HttpServlet {

    private static String PACKAGE_NAME = "controllers.ueat.exp";

    private static final long serialVersionUID = 1L;

    /**
     * @see HttpServlet#HttpServlet()
     */
    public EXPMainServlet() {
        super();
    }

    @Override
    public void init() throws ServletException {
        COMRequestManager.getInstance().initialize(getServletConfig().getServletContext());
        SALBasicData.getInstance().initialize();
    }

    /**
     * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
     */
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        doPost(request, response);
    }

    /**
     * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
     */
    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        HttpSession session = request.getSession(true);
        try {
            if (COMUtility.isNullOrEmpty(request.getParameter("request"))) {
//                if (!comfirm(request, session)) {
                    response.sendRedirect("/ueat/view/jsp/exp/EXPLogin.jsp");
                    return;
//                }
            } else {
//                Object status = session.getAttribute("login_status");
//                if (status == null){
//                    response.sendRedirect("/ueat/view/jsp/exp/EXPLogin.jsp");
//                    return;
//                }
            }

            String nextPage = "";
            String packageName = COMRequestManager.getInstance().getPackageName(request.getParameter("request"));
            if (COMUtility.isNullOrEmpty(packageName)) {
                packageName = PACKAGE_NAME;
            }
            String managerName = COMRequestManager.getInstance().getManagerName(request.getParameter("request"));
            Class<?> targetRequest = Class.forName(packageName + "." + managerName);
            EXPManagerInterface expManager = (EXPManagerInterface)targetRequest.newInstance();
            expManager.setRequest(request);
            expManager.setResponse(response);
            expManager.setServletContext(getServletContext());
            expManager.setSession(session);

            if (expManager.execute()) {
                nextPage = COMRequestManager.getInstance().getNextPage(request.getParameter("request"));
            } else {
                nextPage = "/view/jsp/exp/EXPMain.jsp";
            }

            session.setAttribute("value_bean", expManager.getValueBean());

            RequestDispatcher rd = getServletConfig().getServletContext().getRequestDispatcher(nextPage);
            rd.forward(request, response);
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
            session.invalidate();
            response.sendRedirect("/ueat");
        } catch (InstantiationException e) {
            e.printStackTrace();
            session.invalidate();
            response.sendRedirect("/ueat");
        } catch (IllegalAccessException e) {
            e.printStackTrace();
            session.invalidate();
            response.sendRedirect("/ueat");
        } catch (Exception e) {
            e.printStackTrace();
            session.invalidate();
            response.sendRedirect("/ueat");
        }
    }

    private boolean comfirm(HttpServletRequest request, HttpSession session) {
        boolean ret = false;

        EXPAuthManager authMgr = new EXPAuthManager();
        ret = authMgr.confirmLogin();
        if (!ret) {
            EXPCommonValueBean commValue = (EXPCommonValueBean)authMgr.getValueBean();
            session.setAttribute("msg_type", commValue.getNotifyType());
            session.setAttribute("msg_content", commValue.getNotifyMessage());
        }

        return ret;
    }

}
