package junit.ueat;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import javax.naming.Context;
import javax.naming.InitialContext;

import models.ext.sal.SALPayHistoryValueBean;
import models.ueat.com.COMRequestDefValue;
import models.ueat.exp.EXPHighChartValueBean;
import models.ueat.exp.EXPUtilityExpensesValueBean;

import org.dom4j.DocumentException;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import util.com.COMDataDump;
import util.com.COMRequestDefReader;

import com.mysql.jdbc.jdbc2.optional.MysqlDataSource;

import controllers.ext.sal.SALPayStatementManager;
import controllers.ueat.exp.EXPElectricityBillManager;
import controllers.ueat.exp.EXPManagerInterface;
import controllers.ueat.exp.EXPStructHighChartData;

public class UeatServerSideTest {
    private static List PackageList;

    @SuppressWarnings("unchecked")
    @BeforeClass
    public static void setUpBeforeClass() throws Exception {

        PackageList = new ArrayList<String>();
        PackageList.add("controllers.ueat.exp");
        PackageList.add("controllers.ext.sal");
        PackageList.add("junit.ueat");
        PackageList.add("models.ueat.dba");
        PackageList.add("models.ueat.exp");
        PackageList.add("models.ueat.com");
        PackageList.add("models.ext.dba");
        PackageList.add("models.ext.sal");
    }

    @BeforeClass
    public static void initJndi() throws Exception {
        System.setProperty(Context.INITIAL_CONTEXT_FACTORY, "org.apache.naming.java.javaURLContextFactory");
        System.setProperty(Context.URL_PKG_PREFIXES, "org.apache.naming");
        InitialContext ic = new InitialContext();
        ic.createSubcontext("java:");
        ic.createSubcontext("java:comp");
        ic.createSubcontext("java:comp/env");
        ic.createSubcontext("java:comp/env/jdbc");
        MysqlDataSource ds = new MysqlDataSource();
        ds.setUser("fuji0");
        ds.setPassword("fujimaru");
        // ds.setURL("jdbc:mys\ql://192.168.11.11:3306/ueat");
         ds.setURL("jdbc:mysql://192.168.192.128:3306/ueat");
        ic.bind("java:comp/env/jdbc/ueat", ds);
    }

    @AfterClass
    public static void tearDownAfterClass() throws Exception {
    }

    @Before
    public void setUp() throws Exception {
    }

    @After
    public void tearDown() throws Exception {
    }

    @Test
    public void test() {
        try {
            EXPManagerInterface ebManager = EXPElectricityBillManager.class.newInstance();
            ebManager.execute();
            EXPUtilityExpensesValueBean value = (EXPUtilityExpensesValueBean) ebManager.getValueBean();

            EXPStructHighChartData chData = new EXPStructHighChartData();
            chData.setHighChartValue(new EXPHighChartValueBean());
            chData.setTargetValue(value);
            String strChartData = chData.createBillChartData();

            COMDataDump dump = new COMDataDump();
            dump.setTargetCategoryFromPackage(PackageList);

            Field field = EXPUtilityExpensesValueBean.class.getDeclaredField("ExpEBValueBeanList");
            dump.dumpData(value, field);

            COMRequestDefReader xmlReader = new COMRequestDefReader();
            // File file = new File(new File(".").getAbsolutePath() +
            // "/def/RequestDef.xml");
            File file = new File("./def/RequestDef.xml");
            if (file.exists()) {
                List reqDefValue = new ArrayList<COMRequestDefValue>();
                xmlReader.readRequestDef(file.getPath(), reqDefValue);
                Field[] fields = COMRequestDefValue.class.getDeclaredFields();
                for (int i = 0; i < reqDefValue.size(); i++) {
                    dump.dumpData((COMRequestDefValue) reqDefValue.get(i), fields);
                }
            }

            EXPManagerInterface psManager = SALPayStatementManager.class.newInstance();
            psManager.execute();
            SALPayHistoryValueBean phValue = (SALPayHistoryValueBean) psManager.getValueBean();

            field = SALPayHistoryValueBean.class.getDeclaredField("PayStatementList");
            dump.dumpData(phValue, field);
        }
        catch (InstantiationException | IllegalAccessException e) {
            e.printStackTrace();
        }
        catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
        catch (IOException e) {
            e.printStackTrace();
        }
        catch (NoSuchFieldException e) {
            e.printStackTrace();
        }
        catch (SecurityException e) {
            e.printStackTrace();
        }
        catch (DocumentException e) {
            e.printStackTrace();
        }
    }
}
