package models.ext.dba.bdy;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;

import models.ext.bdy.BDYHealthValueBean;
import models.ext.bdy.BDYWeightValueBean;
import models.ueat.dba.DBACommonControlBean;
import models.ueat.dba.DBAControlBeanInterface;
import models.ueat.exp.EXPConditionValueBean;
import util.com.COMDef;
import util.com.COMUtility;

public class DBAWeightRecordControlBean
    extends DBACommonControlBean implements DBAControlBeanInterface {

    private static String DBTBL_NAME_WEIGHT = "weight_record";

    private EXPConditionValueBean CondValue;
    private BDYHealthValueBean TargetValue;
    private BDYHealthValueBean ResultValue;

    @Override
    public Object getCondParam() {
        if (CondValue == null) {
            return null;
        } else {
            return (EXPConditionValueBean)CondValue;
        }
    }

    @Override
    public void setCondParam(Object cond) {
        CondValue = null;
        if (cond != null) {
            CondValue = (EXPConditionValueBean)cond;
        }
    }

    @Override
    public Object getTargetValue() {
        if (TargetValue == null) {
            return null;
        } else {
            return (BDYHealthValueBean)TargetValue;
        }
    }

    @Override
    public void setTargetValue(Object prevValue) {
        TargetValue = null;
        if (prevValue != null) {
            TargetValue = (BDYHealthValueBean)prevValue;
        }
    }

    @Override
    public Object getResultValue() {
        if (ResultValue == null) {
            return null;
        } else {
            return (BDYHealthValueBean)ResultValue;
        }
    }

    @Override
    public void setResultValue(Object nextValue) {
        ResultValue = null;
        if (nextValue != null) {
            ResultValue = (BDYHealthValueBean)nextValue;
        }
    }

    @Override
    public boolean select() {
        try {
            Connection conn = getConnection();

            StringBuffer sql = new StringBuffer();
            sql.append("SELECT ");
            sql.append("    * ");
            sql.append("FROM ");
            sql.append("    " + DBTBL_NAME_WEIGHT);

            String cond = createWhereSql((EXPConditionValueBean)getCondParam());
            if (!COMUtility.isNullOrEmpty(cond)) {
                sql.append(cond);
            }

            PreparedStatement prepStmt = conn.prepareStatement(sql.toString());
            ResultSet rs = prepStmt.executeQuery();

            BDYHealthValueBean nextValue = new BDYHealthValueBean();
            while (rs.next()) {
                BDYWeightValueBean wtalue = new BDYWeightValueBean();
                wtalue.setMeasDate(rs.getString("meas_date"));
                wtalue.setWeight(rs.getDouble("weight"));
                wtalue.setBMI(rs.getDouble("bmi"));
                wtalue.setBodyFat(rs.getDouble("body_fat"));
                wtalue.setVisceralFat(rs.getDouble("visceral_fat"));
                wtalue.setMuscleMass(rs.getDouble("muscle_mass"));
                wtalue.setBasalMetabolism(rs.getDouble("basal_metabolism"));
                nextValue.getWeightValueList().add(wtalue);
            }
            setResultValue(nextValue);

            conn.close();
        }
        catch (SQLException e) {
            return false;
        }

        return true;
    }

    @Override
    public boolean insert() {
        try {
            if (getTargetValue() == null) {
                return false;
            }

            Connection conn = getConnection();
            PreparedStatement ps = null;
            int num = 0;
            String sql = "INSERT INTO " + DBTBL_NAME_WEIGHT + " VALUES (?, ?, ?, ?, ?, ?, ?)";

            BDYHealthValueBean value = (BDYHealthValueBean)getTargetValue();
            for (int i = 0; i < value.getWeightValueList().size(); i++) {
                BDYWeightValueBean wtValue = (BDYWeightValueBean)value.getWeightValueList().get(i);
                ps = conn.prepareStatement(sql);
                ps.setDate(1, COMUtility.getSqlDate(wtValue.getMeasDate()));
                ps.setDouble(2, wtValue.getWeight());
                ps.setDouble(3, wtValue.getBMI());
                ps.setDouble(4, wtValue.getBodyFat());
                ps.setDouble(5, wtValue.getVisceralFat());
                ps.setDouble(6, wtValue.getMuscleMass());
                ps.setDouble(7, wtValue.getBasalMetabolism());
                num += ps.executeUpdate();
            }
            value.setNotifyType(COMDef.NotifyType.INF);
            value.setNotifyMessage("insert count : " + String.valueOf(num));
            setResultValue(value);

            conn.close();
        }
        catch (SQLException e) {
            e.printStackTrace();
            return false;
        }
        catch (ParseException e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

    @Override
    public boolean update() {
        return false;
    }

    @Override
    public boolean delete() {
        return false;
    }

}
