package models.ext.dba.sal;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;

import models.ext.sal.SALAttendValueBean;
import models.ueat.dba.DBACommonControlBean;
import models.ueat.dba.DBAControlBeanInterface;
import models.ueat.exp.EXPConditionValueBean;
import util.com.COMDef;
import util.com.COMUtility;

public class DBAAttendControlBean
    extends DBACommonControlBean implements DBAControlBeanInterface {

    private static String DBTBL_NAME_WORKTIME = "attnd_worktime";
    private static String DBTBL_NAME_VIOLATION = "attnd_violation";
    private static String DBTBL_NAME_HOLIDAY = "attnd_holiday";

    private EXPConditionValueBean CondValue;
    private SALAttendValueBean TargetValue;
    private SALAttendValueBean ResultValue;

    @Override
    public Object getCondParam() {
        if (CondValue == null) {
            return null;
        } else {
            return (EXPConditionValueBean)CondValue;
        }
    }

    @Override
    public void setCondParam(Object cond) {
        CondValue = null;
        if (cond != null) {
            CondValue = (EXPConditionValueBean)cond;
        }
    }

    @Override
    public Object getTargetValue() {
        if (TargetValue == null) {
            return null;
        } else {
            return (SALAttendValueBean)TargetValue;
        }
    }

    @Override
    public void setTargetValue(Object prevValue) {
        TargetValue = null;
        if (prevValue != null) {
            TargetValue = (SALAttendValueBean)prevValue;
        }
    }

    @Override
    public Object getResultValue() {
        if (ResultValue == null) {
            return null;
        } else {
            return (SALAttendValueBean)ResultValue;
        }
    }

    @Override
    public void setResultValue(Object nextValue) {
        ResultValue = null;
        if (nextValue != null) {
            ResultValue = (SALAttendValueBean)nextValue;
        }
    }

    @Override
    public boolean select() {

        try {
            StringBuffer sql;
            PreparedStatement prepStmt;
            ResultSet rs;
            String cond = createWhereSql((EXPConditionValueBean)getCondParam());

            SALAttendValueBean nextValue = new SALAttendValueBean();

            Connection conn = getConnection();

            // ------------------------------------------------------
            // select attnd_worktime
            sql = new StringBuffer();
            sql.append("SELECT ");
            sql.append("    * ");
            sql.append("FROM ");
            sql.append("    " + DBTBL_NAME_WORKTIME);
            if (!COMUtility.isNullOrEmpty(cond)) {
                sql.append(cond);
            }

            prepStmt = conn.prepareStatement(sql.toString());
            rs = prepStmt.executeQuery();

            while (rs.next()) {
                nextValue.setOfficialTime(rs.getDouble("official_time"));
                nextValue.setOverTime(rs.getDouble("over_time"));
                nextValue.setMidnightTime(rs.getDouble("midnight_time"));
                nextValue.setLegalHolidayTime(rs.getDouble("legal_holiday_time"));
            }

            // ------------------------------------------------------
            // select attnd_violation
            sql = new StringBuffer();
            sql.append("SELECT ");
            sql.append("    * ");
            sql.append("FROM ");
            sql.append("    " + DBTBL_NAME_VIOLATION);
            if (!COMUtility.isNullOrEmpty(cond)) {
                sql.append(cond);
            }

            prepStmt = conn.prepareStatement(sql.toString());
            rs = prepStmt.executeQuery();

            while (rs.next()) {
                nextValue.setLateEarlyCount(rs.getInt("late_early_count"));
                nextValue.setLateEarlyTime(rs.getDouble("late_early_time"));
                nextValue.setDeductTime(rs.getDouble("deduct_time"));
                nextValue.setAbsentDays(rs.getInt("absent_days"));
            }

            // ------------------------------------------------------
            // select attnd_holiday
            sql = new StringBuffer();
            sql.append("SELECT ");
            sql.append("    * ");
            sql.append("FROM ");
            sql.append("    " + DBTBL_NAME_HOLIDAY);
            if (!COMUtility.isNullOrEmpty(cond)) {
                sql.append(cond);
            }

            prepStmt = conn.prepareStatement(sql.toString());
            rs = prepStmt.executeQuery();

            while (rs.next()) {
                nextValue.setSpecialDays(rs.getInt("special_days"));
                nextValue.setPaidUsedDays(rs.getInt("paid_used_days"));
                nextValue.setPaidRemainOfLastYear(rs.getInt("paid_remain_of_last_year"));
                nextValue.setPaidRemain(rs.getInt("paid_remain"));
            }

            conn.close();

            setResultValue(nextValue);
        }
        catch (SQLException e) {
            return false;
        }

        return true;
    }

    @Override
    public boolean insert() {
        try {
            if (getTargetValue() == null) {
                return false;
            }

            Connection conn = getKeepConnection();
            String notifyMsg = "";
            boolean isNewConnect = false;
            if (conn == null) {
                conn = getConnection();
                isNewConnect = true;
            }
            PreparedStatement ps = null;
            SALAttendValueBean value = (SALAttendValueBean)getTargetValue();

            // ------------------------------------------------------
            // insert attnd_worktime
            int num = 0;
            String sql = "INSERT INTO " + DBTBL_NAME_WORKTIME + " VALUES (?, ?, ?, ?, ?)";

            ps = conn.prepareStatement(sql);
            ps.setDate(1, COMUtility.getSqlDate(getOccuredTime()));
            ps.setDouble(2, value.getOfficialTime());
            ps.setDouble(3, value.getOverTime());
            ps.setDouble(4, value.getMidnightTime());
            ps.setDouble(5, value.getLegalHolidayTime());
            num += ps.executeUpdate();
            notifyMsg += DBTBL_NAME_WORKTIME + " - insert count : " + String.valueOf(num) + "\n";

            // ------------------------------------------------------
            // insert attnd_violation
            num = 0;
            sql = "INSERT INTO " + DBTBL_NAME_VIOLATION + " VALUES (?, ?, ?, ?, ?)";

            ps = conn.prepareStatement(sql);
            ps.setDate(1, COMUtility.getSqlDate(getOccuredTime()));
            ps.setInt(2, value.getLateEarlyCount());
            ps.setDouble(3, value.getLateEarlyTime());
            ps.setDouble(4, value.getDeductTime());
            ps.setInt(5, value.getAbsentDays());
            num += ps.executeUpdate();
            notifyMsg += DBTBL_NAME_VIOLATION + " - insert count : " + String.valueOf(num) + "\n";

            // ------------------------------------------------------
            // insert attnd_violation
            num = 0;
            sql = "INSERT INTO " + DBTBL_NAME_HOLIDAY + " VALUES (?, ?, ?, ?, ?)";

            ps = conn.prepareStatement(sql);
            ps.setDate(1, COMUtility.getSqlDate(getOccuredTime()));
            ps.setInt(2, value.getSpecialDays());
            ps.setInt(3, value.getPaidUsedDays());
            ps.setInt(4, value.getPaidRemainOfLastYear());
            ps.setInt(5, value.getPaidRemain());
            num += ps.executeUpdate();
            notifyMsg += DBTBL_NAME_HOLIDAY + " - insert count : " + String.valueOf(num) + "\n";

            // ------------------------------------------------------
            value.setNotifyType(COMDef.NotifyType.INF);
            value.setNotifyMessage(notifyMsg);
            setResultValue(value);

            if (isNewConnect) {
                conn.close();
            }
        }
        catch (SQLException e) {
            e.printStackTrace();
            return false;
        }
        catch (ParseException e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

    @Override
    public boolean update() {
        // TODO 自動生成されたメソッド・スタブ
        return false;
    }

    @Override
    public boolean delete() {
        // TODO 自動生成されたメソッド・スタブ
        return false;
    }
}
