package models.ext.dba.sal;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;

import models.ext.sal.SALDeductSocialInsuranceValueBean;
import models.ext.sal.SALDeductTaxValueBean;
import models.ext.sal.SALDeductValueBean;
import models.ueat.dba.DBACommonControlBean;
import models.ueat.dba.DBAControlBeanInterface;
import models.ueat.exp.EXPConditionValueBean;
import util.com.COMDef;
import util.com.COMUtility;

public class DBADeductControlBean
    extends DBACommonControlBean implements DBAControlBeanInterface {

    private static String DBTBL_NAME = "deduct";
    private static String DBTBL_NAME_INSURANCE = "deduct_social_insurance";
    private static String DBTBL_NAME_TAX = "deduct_tax";

    private EXPConditionValueBean CondValue;
    private SALDeductValueBean TargetValue;
    private SALDeductValueBean ResultValue;

    @Override
    public Object getCondParam() {
        if (CondValue == null) {
            return null;
        } else {
            return (EXPConditionValueBean)CondValue;
        }
    }

    @Override
    public void setCondParam(Object cond) {
        CondValue = null;
        if (cond != null) {
            CondValue = (EXPConditionValueBean)cond;
        }
    }

    @Override
    public Object getTargetValue() {
        if (TargetValue == null) {
            return null;
        } else {
            return (SALDeductValueBean)TargetValue;
        }
    }

    @Override
    public void setTargetValue(Object prevValue) {
        TargetValue = null;
        if (prevValue != null) {
            TargetValue = (SALDeductValueBean)prevValue;
        }
    }

    @Override
    public Object getResultValue() {
        if (ResultValue == null) {
            return null;
        } else {
            return (SALDeductValueBean)ResultValue;
        }
    }

    @Override
    public void setResultValue(Object nextValue) {
        ResultValue = null;
        if (nextValue != null) {
            ResultValue = (SALDeductValueBean)nextValue;
        }
    }

    @Override
    public boolean select() {

        try {
            StringBuffer sql;
            PreparedStatement prepStmt;
            ResultSet rs;
            String cond = createWhereSql((EXPConditionValueBean)getCondParam());

            SALDeductValueBean nextValue = new SALDeductValueBean();

            Connection conn = getConnection();

            // ------------------------------------------------------
            // select deduct
            sql = new StringBuffer();
            sql.append("SELECT ");
            sql.append("    * ");
            sql.append("FROM ");
            sql.append("    " + DBTBL_NAME);
            if (!COMUtility.isNullOrEmpty(cond)) {
                sql.append(cond);
            }

            prepStmt = conn.prepareStatement(sql.toString());
            rs = prepStmt.executeQuery();

            while (rs.next()) {
                nextValue.setSocialInsurance(rs.getInt("social_insurance"));
                nextValue.setTax(rs.getInt("tax"));
                nextValue.setDues(rs.getInt("dues"));
                nextValue.setExamFee(rs.getInt("exam_fee"));
                nextValue.setYearEndAdjustResult(rs.getInt("year_end_adjust_result"));
            }

            // ------------------------------------------------------
            // select deduct_social_insurance
            sql = new StringBuffer();
            sql.append("SELECT ");
            sql.append("    * ");
            sql.append("FROM ");
            sql.append("    " + DBTBL_NAME_INSURANCE);
            if (!COMUtility.isNullOrEmpty(cond)) {
                sql.append(cond);
            }

            prepStmt = conn.prepareStatement(sql.toString());
            rs = prepStmt.executeQuery();

            while (rs.next()) {
                SALDeductSocialInsuranceValueBean insuranceValue = new SALDeductSocialInsuranceValueBean();
                insuranceValue.setHealthInsurance(rs.getInt("health_insurance"));
                insuranceValue.setNursingInsurance(rs.getInt("nursing_insurance"));
                insuranceValue.setEmployeePension(rs.getInt("employee_pension"));
                insuranceValue.setEmploymentInsurance(rs.getInt("employment_insurance"));
                insuranceValue.setHarmonization(rs.getInt("harmonization"));
                nextValue.setSocialInsuranceValue(insuranceValue);
            }

            // ------------------------------------------------------
            // select deduct_tax
            sql = new StringBuffer();
            sql.append("SELECT ");
            sql.append("    * ");
            sql.append("FROM ");
            sql.append("    " + DBTBL_NAME_TAX);
            if (!COMUtility.isNullOrEmpty(cond)) {
                sql.append(cond);
            }

            prepStmt = conn.prepareStatement(sql.toString());
            rs = prepStmt.executeQuery();

            while (rs.next()) {
                SALDeductTaxValueBean taxValue = new SALDeductTaxValueBean();
                taxValue.setIncomeTax(rs.getInt("income_tax"));
                taxValue.setInhabitantTax(rs.getInt("inhabitant_tax"));
                taxValue.setNetTaxableAmount(rs.getInt("net_taxable_amount"));
                nextValue.setTaxValue(taxValue);
            }

            conn.close();

            setResultValue(nextValue);
        }
        catch (SQLException e) {
            return false;
        }

        return true;
    }

    @Override
    public boolean insert() {
        try {
            if (getTargetValue() == null) {
                return false;
            }

            Connection conn = getKeepConnection();
            String notifyMsg = "";
            boolean isNewConnect = false;
            if (conn == null) {
                conn = getConnection();
                isNewConnect = true;
            }
            PreparedStatement ps = null;
            SALDeductValueBean value = (SALDeductValueBean)getTargetValue();

            // ------------------------------------------------------
            // insert deduct
            int num = 0;
            String sql = "INSERT INTO " + DBTBL_NAME + " VALUES (?, ?, ?, ?, ?, ?)";

            ps = conn.prepareStatement(sql);
            ps.setDate(1, COMUtility.getSqlDate(getOccuredTime()));
            ps.setInt(2, value.getSocialInsurance());
            ps.setInt(3, value.getTax());
            ps.setInt(4, value.getDues());
            ps.setInt(5, value.getExamFee());
            ps.setInt(6, value.getYearEndAdjustResult());
            num += ps.executeUpdate();
            notifyMsg += DBTBL_NAME + " - insert count : " + String.valueOf(num) + "\n";

            // ------------------------------------------------------
            // insert deduct_social_insurance
            num = 0;
            sql = "INSERT INTO " + DBTBL_NAME_INSURANCE + " VALUES (?, ?, ?, ?, ?, ?)";

            ps = conn.prepareStatement(sql);
            ps.setDate(1, COMUtility.getSqlDate(getOccuredTime()));
            ps.setInt(2, value.getSocialInsuranceValue().getHealthInsurance());
            ps.setInt(3, value.getSocialInsuranceValue().getNursingInsurance());
            ps.setInt(4, value.getSocialInsuranceValue().getEmployeePension());
            ps.setInt(5, value.getSocialInsuranceValue().getEmploymentInsurance());
            ps.setInt(6, value.getSocialInsuranceValue().getHarmonization());
            num += ps.executeUpdate();
            notifyMsg += DBTBL_NAME_INSURANCE + " - insert count : " + String.valueOf(num) + "\n";

            // ------------------------------------------------------
            // insert deduct_tax
            num = 0;
            sql = "INSERT INTO " + DBTBL_NAME_TAX + " VALUES (?, ?, ?, ?)";

            ps = conn.prepareStatement(sql);
            ps.setDate(1, COMUtility.getSqlDate(getOccuredTime()));
            ps.setInt(2, value.getTaxValue().getIncomeTax());
            ps.setInt(3, value.getTaxValue().getInhabitantTax());
            ps.setInt(4, value.getTaxValue().getNetTaxableAmount());
            num += ps.executeUpdate();
            notifyMsg += DBTBL_NAME_TAX + " - insert count : " + String.valueOf(num) + "\n";

            // ------------------------------------------------------
            value.setNotifyType(COMDef.NotifyType.INF);
            value.setNotifyMessage(notifyMsg);
            setResultValue(value);

            if (isNewConnect) {
                conn.close();
            }
        }
        catch (SQLException e) {
            e.printStackTrace();
            return false;
        }
        catch (ParseException e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

    @Override
    public boolean update() {
        // TODO 自動生成されたメソッド・スタブ
        return false;
    }

    @Override
    public boolean delete() {
        // TODO 自動生成されたメソッド・スタブ
        return false;
    }
}
