package models.ext.dba.sal;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;

import models.ext.sal.SALPaidBasicValueBean;
import models.ext.sal.SALPaidValueBean;
import models.ueat.dba.DBACommonControlBean;
import models.ueat.dba.DBAControlBeanInterface;
import models.ueat.exp.EXPConditionValueBean;
import util.com.COMDef;
import util.com.COMUtility;

public class DBAPaidControlBean
    extends DBACommonControlBean implements DBAControlBeanInterface {

    private static String DBTBL_NAME_SALARY = "paid_salary";
    private static String DBTBL_NAME_BASIC = "paid_basic";

    private EXPConditionValueBean CondValue;
    private SALPaidValueBean TargetValue;
    private SALPaidValueBean ResultValue;

    @Override
    public Object getCondParam() {
        if (CondValue == null) {
            return null;
        } else {
            return (EXPConditionValueBean)CondValue;
        }
    }

    @Override
    public void setCondParam(Object cond) {
        CondValue = null;
        if (cond != null) {
            CondValue = (EXPConditionValueBean)cond;
        }
    }

    @Override
    public Object getTargetValue() {
        if (TargetValue == null) {
            return null;
        } else {
            return (SALPaidValueBean)TargetValue;
        }
    }

    @Override
    public void setTargetValue(Object prevValue) {
        TargetValue = null;
        if (prevValue != null) {
            TargetValue = (SALPaidValueBean)prevValue;
        }
    }

    @Override
    public Object getResultValue() {
        if (ResultValue == null) {
            return null;
        } else {
            return (SALPaidValueBean)ResultValue;
        }
    }

    @Override
    public void setResultValue(Object nextValue) {
        ResultValue = null;
        if (nextValue != null) {
            ResultValue = (SALPaidValueBean)nextValue;
        }
    }

    @Override
    public boolean select() {

        try {
            StringBuffer sql;
            PreparedStatement prepStmt;
            ResultSet rs;
            String cond = createWhereSql((EXPConditionValueBean)getCondParam());

            SALPaidValueBean nextValue = new SALPaidValueBean();
            SALPaidBasicValueBean basicValue = new SALPaidBasicValueBean();

            Connection conn = getConnection();

            // ------------------------------------------------------
            // select paid_salary
            sql = new StringBuffer();
            sql.append("SELECT ");
            sql.append("    * ");
            sql.append("FROM ");
            sql.append("    " + DBTBL_NAME_SALARY);
            if (!COMUtility.isNullOrEmpty(cond)) {
                sql.append(cond);
            }

            prepStmt = conn.prepareStatement(sql.toString());
            rs = prepStmt.executeQuery();

            while (rs.next()) {
                nextValue.setLegalHolidayAllowance(rs.getInt("legal_holiday_allowance"));
                nextValue.setCommuteAllowance(rs.getInt("commute_allowance"));
                nextValue.setCommuteAllowanceDiff(rs.getInt("commute_allowance_diff"));
                nextValue.setSpecialAllowance(rs.getInt("special_allowance"));
                nextValue.setOverAllowance(rs.getInt("over_allowance"));
                nextValue.setMidnightAllowance(rs.getInt("midnight_allowance"));
                nextValue.setViolationDeduct(rs.getInt("violation_deduct"));
                nextValue.setRetroactivityPaid(rs.getInt("retroactivity_paid"));

                basicValue.setBasicId(rs.getInt("basic_id"));
            }

            conn.close();

            // ------------------------------------------------------
            // select paid_basic
            EXPConditionValueBean basicCondValue = new EXPConditionValueBean();
            basicCondValue.setWhereCond(" basic_id = " + String.valueOf(basicValue.getBasicId()));

            selectPaidBasic(basicValue, basicCondValue);
            nextValue.setPaidBasic(basicValue);

            setResultValue(nextValue);
        }
        catch (SQLException e) {
            e.printStackTrace();
            return false;
        }

        return true;
    }

    public boolean selectPaidBasic(SALPaidBasicValueBean basicValue, EXPConditionValueBean condValue) {
        try {
            StringBuffer sql;
            PreparedStatement prepStmt;
            ResultSet rs;

            if (basicValue == null) {
                return false;
            }

            Connection conn = getConnection();

            sql = new StringBuffer();
            sql.append("SELECT ");
            sql.append("    * ");
            sql.append("FROM ");
            sql.append("    " + DBTBL_NAME_BASIC);
            String basicCond = createWhereSql(condValue);
            if (!COMUtility.isNullOrEmpty(basicCond)) {
                sql.append(basicCond);
            }

            prepStmt = conn.prepareStatement(sql.toString());
            rs = prepStmt.executeQuery();

            while (rs.next()) {
                basicValue.setBasicId(rs.getInt("basic_id"));
                basicValue.setRevisionDate(rs.getString("revision_date"));
                basicValue.setBaseSalary(rs.getInt("base_salary"));
                basicValue.setExecutiveAllowance(rs.getInt("executive_allowance"));
                basicValue.setTechnologyAllowance(rs.getInt("technology_allowance"));
                basicValue.setBusinessAllowance(rs.getInt("business_allowance"));
                basicValue.setCapacityAllowance(rs.getInt("capacity_allowance"));
                basicValue.setHousingAllowance(rs.getInt("housing_allowance"));
                basicValue.setFamilyAllowance(rs.getInt("family_allowance"));
                basicValue.setSalesAllowance(rs.getInt("sales_allowance"));
            }

            conn.close();
        }
        catch (SQLException e) {
            e.printStackTrace();
            return false;
        }

        return true;
    }

    @Override
    public boolean insert() {
        try {
            if (getTargetValue() == null) {
                return false;
            }

            Connection conn = getKeepConnection();
            String notifyMsg = "";
            boolean isNewConnect = false;
            if (conn == null) {
                conn = getConnection();
                isNewConnect = true;
            }
            PreparedStatement ps = null;
            SALPaidValueBean value = (SALPaidValueBean)getTargetValue();

            // ------------------------------------------------------
            // insert attnd_worktime
            int num = 0;
            String sql = "INSERT INTO " + DBTBL_NAME_SALARY + " VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";

            ps = conn.prepareStatement(sql);
            ps.setDate(1, COMUtility.getSqlDate(getOccuredTime()));
            ps.setInt(2, value.getPaidBasic().getBasicId());
            ps.setInt(3, value.getLegalHolidayAllowance());
            ps.setInt(4, value.getCommuteAllowance());
            ps.setInt(5, value.getCommuteAllowanceDiff());
            ps.setInt(6, value.getSpecialAllowance());
            ps.setInt(7, value.getOverAllowance());
            ps.setInt(8, value.getMidnightAllowance());
            ps.setInt(9, value.getViolationDeduct());
            ps.setInt(10, value.getRetroactivityPaid());
            ps.setInt(11, value.getBonus());
            num += ps.executeUpdate();
            notifyMsg += DBTBL_NAME_SALARY + " - insert count : " + String.valueOf(num) + "\n";

            // ------------------------------------------------------
            value.setNotifyType(COMDef.NotifyType.INF);
            value.setNotifyMessage(notifyMsg);
            setResultValue(value);

            if (isNewConnect) {
                conn.close();
            }
        }
        catch (SQLException e) {
            e.printStackTrace();
            return false;
        }
        catch (ParseException e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

    @Override
    public boolean update() {
        // TODO 自動生成されたメソッド・スタブ
        return false;
    }

    @Override
    public boolean delete() {
        // TODO 自動生成されたメソッド・スタブ
        return false;
    }
}
