package models.ext.dba.sal;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;

import models.ext.sal.SALAttendValueBean;
import models.ext.sal.SALDeductValueBean;
import models.ext.sal.SALPaidValueBean;
import models.ext.sal.SALPayHistoryValueBean;
import models.ext.sal.SALPayStatementValueBean;
import models.ueat.dba.DBACommonControlBean;
import models.ueat.dba.DBAControlBeanInterface;
import models.ueat.exp.EXPConditionValueBean;
import util.com.COMDef;
import util.com.COMUtility;

public class DBAPayStatementControlBean
    extends DBACommonControlBean implements DBAControlBeanInterface {

    private static String DBTBL_NAME = "pay_statement";

    private EXPConditionValueBean CondValue;
    private SALPayHistoryValueBean TargetValue;
    private SALPayHistoryValueBean ResultValue;

    @Override
    public Object getCondParam() {
        if (CondValue == null) {
            return null;
        } else {
            return (EXPConditionValueBean)CondValue;
        }
    }

    @Override
    public void setCondParam(Object cond) {
        CondValue = null;
        if (cond != null) {
            CondValue = (EXPConditionValueBean)cond;
        }
    }

    @Override
    public Object getTargetValue() {
        if (TargetValue == null) {
            return null;
        } else {
            return (SALPayHistoryValueBean)TargetValue;
        }
    }

    @Override
    public void setTargetValue(Object prevValue) {
        TargetValue = null;
        if (prevValue != null) {
            TargetValue = (SALPayHistoryValueBean)prevValue;
        }
    }

    @Override
    public Object getResultValue() {
        if (ResultValue == null) {
            return null;
        } else {
            return (SALPayHistoryValueBean)ResultValue;
        }
    }

    @Override
    public void setResultValue(Object nextValue) {
        ResultValue = null;
        if (nextValue != null) {
            ResultValue = (SALPayHistoryValueBean)nextValue;
        }
    }

    @Override
    public boolean select() {

        try {
            Connection conn = getConnection();

            StringBuffer sql = new StringBuffer();
            sql.append("SELECT ");
            sql.append("    * ");
            sql.append("FROM ");
            sql.append("    " + DBTBL_NAME);

            String cond = createWhereSql((EXPConditionValueBean)getCondParam());
            if (!COMUtility.isNullOrEmpty(cond)) {
                sql.append(cond);
            }

            PreparedStatement prepStmt = conn.prepareStatement(sql.toString());
            ResultSet rs = prepStmt.executeQuery();

            SALPayHistoryValueBean nextValue = new SALPayHistoryValueBean();
            while (rs.next()) {
                SALPayStatementValueBean payStmtValue = new SALPayStatementValueBean();
                payStmtValue.setPayDate(rs.getString("pay_date"));
                payStmtValue.setTotalAmountPaid(rs.getInt("total_amount_paid"));
                payStmtValue.setDeductTotal(rs.getInt("deduct_total"));
                payStmtValue.setNetAmountPaid(rs.getInt("net_amount_paid"));
                payStmtValue.setBonusMode(false);
                if (rs.getInt("bonus_flag") == 1) {
                    payStmtValue.setBonusMode(true);
                }
                nextValue.getPayStatementList().add(payStmtValue);
            }
            conn.close();

            for (int i = 0; i < nextValue.getPayStatementList().size(); i++) {
                SALPayStatementValueBean payStmtValue = (SALPayStatementValueBean)nextValue.getPayStatementList().get(i);

                EXPConditionValueBean condValue = new EXPConditionValueBean();
                condValue.setWhereCond(" pay_date = '" + payStmtValue.getPayDate() + "'");

                if (!payStmtValue.isBonusMode()) {
                    DBAAttendControlBean dbaAttend = new DBAAttendControlBean();
                    dbaAttend.setCondParam(condValue);
                    if (!dbaAttend.select()) {
                        return false;
                    }
                    payStmtValue.setAttendValue((SALAttendValueBean)dbaAttend.getResultValue());
                }

                DBAPaidControlBean dbaPaid = new DBAPaidControlBean();
                dbaPaid.setCondParam(condValue);
                if (!dbaPaid.select()) {
                    return false;
                }
                payStmtValue.setPaidValue((SALPaidValueBean)dbaPaid.getResultValue());

                DBADeductControlBean dbaDeduct = new DBADeductControlBean();
                dbaDeduct.setCondParam(condValue);
                if (!dbaDeduct.select()) {
                    return false;
                }
                payStmtValue.setDeductValue((SALDeductValueBean)dbaDeduct.getResultValue());
            }

            setResultValue(nextValue);
        }
        catch (SQLException e) {
            return false;
        }

        return true;
    }

    @Override
    public boolean insert() {
        Connection conn;
        try {
            if (getTargetValue() == null) {
                return false;
            }

            conn = getConnection();
            conn.setAutoCommit(false);

            String notifyMsg = "";
            PreparedStatement ps = null;
            int num = 0;
            String sql = "INSERT INTO " + DBTBL_NAME + " VALUES (?, ?, ?, ?, ?)";

            SALPayHistoryValueBean value = (SALPayHistoryValueBean)getTargetValue();
            for (int i = 0; i < value.getPayStatementList().size(); i++) {
                SALPayStatementValueBean psValue = (SALPayStatementValueBean)value.getPayStatementList().get(i);
                ps = conn.prepareStatement(sql);
                ps.setDate(1, COMUtility.getSqlDate(psValue.getPayDate()));
                ps.setInt(2, psValue.getTotalAmountPaid());
                ps.setInt(3, psValue.getDeductTotal());
                ps.setInt(4, psValue.getNetAmountPaid());
                if (psValue.isBonusMode()) {
                    ps.setInt(5, 1);
                } else {
                    ps.setInt(5, 0);
                }
                num += ps.executeUpdate();
                notifyMsg += DBTBL_NAME + " - insert count : " + String.valueOf(num) + "\n";

                if (!psValue.isBonusMode()) {
                    // ------------------------------------------------------
                    // insert attend
                    DBAAttendControlBean dbaAttendControl = new DBAAttendControlBean();
                    dbaAttendControl.setKeepConnection(conn);
                    dbaAttendControl.setTargetValue(psValue.getAttendValue());
                    dbaAttendControl.setOccuredTime(psValue.getPayDate());
                    if (!dbaAttendControl.insert()) {
                        return false;
                    }
                    notifyMsg += ((SALAttendValueBean)dbaAttendControl.getResultValue()).getNotifyMessage() + "\n";
                }

                // ------------------------------------------------------
                // insert paid
                DBAPaidControlBean dbaPaidControl = new DBAPaidControlBean();
                dbaPaidControl.setKeepConnection(conn);
                dbaPaidControl.setTargetValue(psValue.getPaidValue());
                dbaPaidControl.setOccuredTime(psValue.getPayDate());
                if (!dbaPaidControl.insert()) {
                    return false;
                }
                notifyMsg += ((SALPaidValueBean)dbaPaidControl.getResultValue()).getNotifyMessage() + "\n";

                // ------------------------------------------------------
                // insert deduct
                DBADeductControlBean dbaDeductControl = new DBADeductControlBean();
                dbaDeductControl.setKeepConnection(conn);
                dbaDeductControl.setTargetValue(psValue.getDeductValue());
                dbaDeductControl.setOccuredTime(psValue.getPayDate());
                if (!dbaDeductControl.insert()) {
                    return false;
                }
                notifyMsg += ((SALDeductValueBean)dbaDeductControl.getResultValue()).getNotifyMessage() + "\n";
            }
            value.setNotifyType(COMDef.NotifyType.INF);
            value.setNotifyMessage(notifyMsg);
            setResultValue(value);

            // commit
            conn.commit();

            conn.close();
        }
        catch (SQLException e) {
            e.printStackTrace();
            return false;
        }
        catch (ParseException e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

    @Override
    public boolean update() {
        // TODO 自動生成されたメソッド・スタブ
        return false;
    }

    @Override
    public boolean delete() {
        // TODO 自動生成されたメソッド・スタブ
        return false;
    }
}
