package models.ext.sal;

import models.ext.dba.sal.DBAPaidControlBean;
import models.ext.dba.sal.DBAPsDesignInfoControlBean;
import models.ueat.exp.EXPConditionValueBean;
import util.com.COMUtility;
import util.ext.sal.SALDef;


public class SALBasicData {
    private static SALBasicData Instance;
    private SALPaidBasicValueBean DefaultPaidBasicValue;
    private SALPaidBasicValueBean NewestPaidBasicValue;
    private SALDesignInfoValueBean PsDesignInfo;

    public static SALBasicData getInstance() {
        if (Instance == null) {
            Instance = new SALBasicData();
        }
        return Instance;
    }

    public void initialize() {
        NewestPaidBasicValue = new SALPaidBasicValueBean();
        DefaultPaidBasicValue = new SALPaidBasicValueBean();
        DBAPaidControlBean paidControl = new DBAPaidControlBean();

        EXPConditionValueBean basicCondValue = new EXPConditionValueBean();
        basicCondValue.setWhereCond(" basic_id = (select max(basic_id) from paid_basic) ");
        paidControl.selectPaidBasic(NewestPaidBasicValue, basicCondValue);

        // for Bonus
        basicCondValue.setWhereCond(" basic_id = (select min(basic_id) from paid_basic) ");
        paidControl.selectPaidBasic(DefaultPaidBasicValue, basicCondValue);

        DBAPsDesignInfoControlBean psDesignControl = new DBAPsDesignInfoControlBean();
        psDesignControl.select();
        PsDesignInfo = (SALDesignInfoValueBean)psDesignControl.getResultValue();
    }

    public SALPaidBasicValueBean getDefaultPaidBasicValue() {
        return DefaultPaidBasicValue;
    }
    public SALPaidBasicValueBean getNewestPaidBasicValue() {
        return NewestPaidBasicValue;
    }

    public SALDesignInfoValueBean getPsDesignInfo() {
        if (NewestPaidBasicValue != null && PsDesignInfo != null) {
            getPsDesignElement(SALDef.DB_COL_BASE_SALARY).setDefaultValue(NewestPaidBasicValue.getBaseSalary());
            getPsDesignElement(SALDef.DB_COL_EXECUTIVE_ALLOWANCE).setDefaultValue(NewestPaidBasicValue.getExecutiveAllowance());
            getPsDesignElement(SALDef.DB_COL_TECHNOLOGY_ALLOWANCE).setDefaultValue(NewestPaidBasicValue.getTechnologyAllowance());
            getPsDesignElement(SALDef.DB_COL_BUSINESS_ALLOWANCE).setDefaultValue(NewestPaidBasicValue.getBusinessAllowance());
            getPsDesignElement(SALDef.DB_COL_CAPACITY_ALLOWANCE).setDefaultValue(NewestPaidBasicValue.getCapacityAllowance());
            getPsDesignElement(SALDef.DB_COL_HOUSING_ALLOWANCE).setDefaultValue(NewestPaidBasicValue.getHousingAllowance());
            getPsDesignElement(SALDef.DB_COL_FAMILY_ALLOWANCE).setDefaultValue(NewestPaidBasicValue.getFamilyAllowance());
            getPsDesignElement(SALDef.DB_COL_SALES_ALLOWANCE).setDefaultValue(NewestPaidBasicValue.getSalesAllowance());
        }

        return PsDesignInfo;
    }

    public SALDesignInfoValueBean.DesignElement getPsDesignElement(String dbColName) {
        if (PsDesignInfo == null || COMUtility.isNullOrEmpty(dbColName)) {
            return null;
        }
        for (int i = 0; i < PsDesignInfo.getDesignList().size(); i++) {
            SALDesignInfoValueBean.DesignElement elem =
                    (SALDesignInfoValueBean.DesignElement)PsDesignInfo.getDesignList().get(i);
            if (dbColName.equals(elem.getDBColName())) {
                return elem;
            }
        }
        return null;
    }
}
