package models.ueat.dba;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;

import models.ueat.exp.EXPConditionValueBean;
import models.ueat.exp.EXPGasBillBValueBean;
import models.ueat.exp.EXPUtilityExpensesValueBean;
import util.com.COMDef;
import util.com.COMUtility;
import util.exp.EXPDef.ExpKind;

public class DBAGasBillControlBean
    extends DBACommonControlBean implements DBAControlBeanInterface {

    private static String DBTBL_NAME = "gas_bill";

    private EXPConditionValueBean CondValue;
    private EXPUtilityExpensesValueBean TargetValue;
    private EXPUtilityExpensesValueBean ResultValue;

    @Override
    public Object getCondParam() {
        if (CondValue == null) {
            return null;
        } else {
            return (EXPConditionValueBean)CondValue;
        }
    }

    @Override
    public void setCondParam(Object cond) {
        CondValue = null;
        if (cond != null) {
            CondValue = (EXPConditionValueBean)cond;
        }
    }

    @Override
    public Object getTargetValue() {
        if (TargetValue == null) {
            return null;
        } else {
            return (EXPUtilityExpensesValueBean)TargetValue;
        }
    }

    @Override
    public void setTargetValue(Object prevValue) {
        TargetValue = null;
        if (prevValue != null) {
            TargetValue = (EXPUtilityExpensesValueBean)prevValue;
        }
    }

    @Override
    public Object getResultValue() {
        if (ResultValue == null) {
            return null;
        } else {
            return (EXPUtilityExpensesValueBean)ResultValue;
        }
    }

    @Override
    public void setResultValue(Object nextValue) {
        ResultValue = null;
        if (nextValue != null) {
            ResultValue = (EXPUtilityExpensesValueBean)nextValue;
        }
    }

    @Override
    public boolean select() {

        try {
            Connection conn = getConnection();

            StringBuffer sql = new StringBuffer();
            sql.append("SELECT ");
            sql.append("    * ");
            sql.append("FROM ");
            sql.append("    " + DBTBL_NAME);

            String cond = createWhereSql((EXPConditionValueBean)getCondParam());
            if (!COMUtility.isNullOrEmpty(cond)) {
                sql.append(cond);
            }

            PreparedStatement prepStmt = conn.prepareStatement(sql.toString());
            ResultSet rs = prepStmt.executeQuery();

            EXPUtilityExpensesValueBean nextValue = new EXPUtilityExpensesValueBean();
            while (rs.next()) {
                EXPGasBillBValueBean gbValue = new EXPGasBillBValueBean();
                gbValue.setId(rs.getInt("id"));
                gbValue.setUsedDate(rs.getString("used_date"));
                gbValue.setGasUsage(rs.getDouble("gas_usage"));
                gbValue.setBasicCharge(rs.getDouble("basic_charge"));
                gbValue.setEnergyCharge(rs.getDouble("energy_charge"));
                gbValue.setPayment(rs.getDouble("payment"));
                nextValue.getExpGBValueBeanList().add(gbValue);
            }
            nextValue.setExpKind(ExpKind.Gas);
            setResultValue(nextValue);

            conn.close();
        }
        catch (SQLException e) {
            return false;
        }

        return true;
    }

    @Override
    public boolean insert() {
        try {
            if (getTargetValue() == null) {
                return false;
            }

            Connection conn = getConnection();
            PreparedStatement ps = null;
            int num = 0;
            String sql = "INSERT INTO " + DBTBL_NAME + " VALUES (?, ?, ?, ?, ?, ?, ?)";

            EXPUtilityExpensesValueBean value = (EXPUtilityExpensesValueBean)getTargetValue();
            for (int i = 0; i < value.getExpGBValueBeanList().size(); i++) {
                EXPGasBillBValueBean gbValue = (EXPGasBillBValueBean)value.getExpGBValueBeanList().get(i);
                ps = conn.prepareStatement(sql);
                ps.setInt(1, issueNewId(DBTBL_NAME));
                ps.setString(2, gbValue.getLoginUserID());
                ps.setDate(3, COMUtility.getSqlDate(gbValue.getUsedDate()));
                ps.setDouble(4, gbValue.getGasUsage());
                ps.setDouble(5, gbValue.getBasicCharge());
                ps.setDouble(6, gbValue.getEnergyCharge());
                ps.setDouble(7, gbValue.getPayment());
                num += ps.executeUpdate();
            }
            value.setExpKind(ExpKind.Gas);
            value.setNotifyType(COMDef.NotifyType.INF);
            value.setNotifyMessage("insert count : " + String.valueOf(num));
            setResultValue(value);

            conn.close();
        }
        catch (SQLException e) {
            e.printStackTrace();
            return false;
        }
        catch (ParseException e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

    @Override
    public boolean update() {
        // TODO 自動生成されたメソッド・スタブ
        return false;
    }

    @Override
    public boolean delete() {
        // TODO 自動生成されたメソッド・スタブ
        return false;
    }
}
