package models.ueat.dba;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;

import models.ueat.exp.EXPConditionValueBean;
import models.ueat.exp.EXPUtilityExpensesValueBean;
import models.ueat.exp.EXPWaterBillBValueBean;
import util.com.COMDef;
import util.com.COMUtility;
import util.exp.EXPDef.ExpKind;

public class DBAWaterBillControlBean
    extends DBACommonControlBean implements DBAControlBeanInterface {

    private static String DBTBL_NAME = "water_bill";

    private EXPConditionValueBean CondValue;
    private EXPUtilityExpensesValueBean TargetValue;
    private EXPUtilityExpensesValueBean ResultValue;

    @Override
    public Object getCondParam() {
        if (CondValue == null) {
            return null;
        } else {
            return (EXPConditionValueBean)CondValue;
        }
    }

    @Override
    public void setCondParam(Object cond) {
        CondValue = null;
        if (cond != null) {
            CondValue = (EXPConditionValueBean)cond;
        }
    }

    @Override
    public Object getTargetValue() {
        if (TargetValue == null) {
            return null;
        } else {
            return (EXPUtilityExpensesValueBean)TargetValue;
        }
    }

    @Override
    public void setTargetValue(Object prevValue) {
        TargetValue = null;
        if (prevValue != null) {
            TargetValue = (EXPUtilityExpensesValueBean)prevValue;
        }
    }

    @Override
    public Object getResultValue() {
        if (ResultValue == null) {
            return null;
        } else {
            return (EXPUtilityExpensesValueBean)ResultValue;
        }
    }

    @Override
    public void setResultValue(Object nextValue) {
        ResultValue = null;
        if (nextValue != null) {
            ResultValue = (EXPUtilityExpensesValueBean)nextValue;
        }
    }

    @Override
    public boolean select() {

        try {
            Connection conn = getConnection();

            StringBuffer sql = new StringBuffer();
            sql.append("SELECT ");
            sql.append("    * ");
            sql.append("FROM ");
            sql.append("    " + DBTBL_NAME);

            String cond = createWhereSql((EXPConditionValueBean)getCondParam());
            if (!COMUtility.isNullOrEmpty(cond)) {
                sql.append(cond);
            }

            PreparedStatement prepStmt = conn.prepareStatement(sql.toString());
            ResultSet rs = prepStmt.executeQuery();

            EXPUtilityExpensesValueBean nextValue = new EXPUtilityExpensesValueBean();
            while (rs.next()) {
                EXPWaterBillBValueBean ebValue = new EXPWaterBillBValueBean();
                ebValue.setId(rs.getInt("id"));
                ebValue.setUsedDate(rs.getString("used_date"));
                ebValue.setWaterUsage(rs.getDouble("water_usage"));
                ebValue.setPayment(rs.getDouble("payment"));
                nextValue.getExpWBValueBeanList().add(ebValue);
            }
            nextValue.setExpKind(ExpKind.Water);
            setResultValue(nextValue);

            conn.close();
        }
        catch (SQLException e) {
            return false;
        }

        return true;
    }

    @Override
    public boolean insert() {
        try {
            if (getTargetValue() == null) {
                return false;
            }

            Connection conn = getConnection();
            PreparedStatement ps = null;
            int num = 0;
            String sql = "INSERT INTO " + DBTBL_NAME + " VALUES (?, ?, ?, ?, ?)";

            EXPUtilityExpensesValueBean value = (EXPUtilityExpensesValueBean)getTargetValue();
            for (int i = 0; i < value.getExpWBValueBeanList().size(); i++) {
                EXPWaterBillBValueBean wbValue = (EXPWaterBillBValueBean)value.getExpWBValueBeanList().get(i);
                ps = conn.prepareStatement(sql);
                ps.setInt(1, issueNewId(DBTBL_NAME));
                ps.setString(2, wbValue.getLoginUserID());
                ps.setDate(3, COMUtility.getSqlDate(wbValue.getUsedDate()));
                ps.setDouble(4, wbValue.getWaterUsage());
                ps.setDouble(5, wbValue.getPayment());
                num += ps.executeUpdate();
            }
            value.setExpKind(ExpKind.Water);
            value.setNotifyType(COMDef.NotifyType.INF);
            value.setNotifyMessage("insert count : " + String.valueOf(num));
            setResultValue(value);

            conn.close();
        }
        catch (SQLException e) {
            e.printStackTrace();
            return false;
        }
        catch (ParseException e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

    @Override
    public boolean update() {
        // TODO 自動生成されたメソッド・スタブ
        return false;
    }

    @Override
    public boolean delete() {
        // TODO 自動生成されたメソッド・スタブ
        return false;
    }
}
