package util.com;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.SecretKeySpec;

public class COMUtility {
    /**
     * 暗号化キーアルゴリズム
     */
    private static final String ALGORITHM_KEY = "AES";
    /**
     * 暗号化アルゴリズム
     */
    private static final String ALGORITHM_CRYPT = "AES/ECB/PKCS5Padding";
    /**
     * 暗号化キー(128ビット)
     */
    private static final String KEY_PASSWORD = "encrypt-key-pass";
    /**
     * エンコーディング
     */
    public static final String ENCODING = "UTF-8";

    public static java.sql.Date getSqlDate(String strDate) throws ParseException {
        strDate = strDate.replaceAll("/", "-");
        strDate = strDate.replaceAll("\\.", "-");
        DateFormat foramt = new SimpleDateFormat("yyyy-MM-dd");
        java.util.Date date = foramt.parse(strDate);

        return new java.sql.Date(date.getTime());
    }

    /**
     * @param parameter iso-8859-1's parameter
     * @return decode parameter
     */
    public static String getUTF8DecodeParameter(String parameter) {
        try {
            return new String(URLDecoder.decode(parameter, "iso-8859-1").getBytes("iso-8859-1"),"utf-8");
        }
        catch (UnsupportedEncodingException e) {
            return "";
        }
    }

    /**
     * @param parameter iso-8859-1's parameter
     * @return decode parameter
     */
    public static String[] getUTF8DecodeParameters(String[] parameter) {
        String[] retParameter = new String[parameter.length];
        try {
            for (int i = 0; i < parameter.length; i++) {
                retParameter[i] = new String(URLDecoder.decode(parameter[i], "iso-8859-1").getBytes("iso-8859-1"),"utf-8");
            }
        }
        catch (UnsupportedEncodingException e) {
            return retParameter;
        }
        return retParameter;
    }

    public static String escapeString(String str) {
        String retVal = "";
        if (str != null) {
            retVal = str.replaceAll("\\r\\n|\\r|\\n", "\\\\r\\\\n");
        }
        return retVal;
    }

    public static String replaceAllRegex(String value, String regex, String replacement) {
        if ( value == null || value.length() == 0 || regex == null || regex.length() == 0 || replacement == null )
            return "";
        return Pattern.compile(regex).matcher(value).replaceAll(replacement);
    }

    public static boolean isNullOrEmpty(String str) {
        // return true, if str is null or empty
        if (str == null || str.length() == 0) {
            return true;
        } else {
            // return true, if str is space only
            String strTrim = str.trim();
            if (strTrim.length() == 0) {
                return true;
            }
        }
        return false;
    }

    public static String getCorrectId(String id) {
        if (id == null) {
            return "0";
        }
        Pattern p = Pattern.compile("^[0-9]+$");
        Matcher m = p.matcher(id);
        if (m.find()) {
            return id;
        }
        return "0";
    }

    public static double getIntParam(String value) {
        try {
            return Integer.parseInt(value);
        } catch (NumberFormatException ex) {
            return 0;
        }
    }

    public static double getDoubleParam(String value) {
        try {
            return Double.parseDouble(value);
        } catch (NumberFormatException ex) {
            return 0;
        }
    }

    /**
     * 暗号化する
     *
     * @param key
     * @param text
     * @return
     * @throws NoSuchAlgorithmException
     * @throws NoSuchPaddingException
     * @throws InvalidKeyException
     * @throws IllegalBlockSizeException
     * @throws BadPaddingException
     * @throws UnsupportedEncodingException
     */
    public static byte[] encrypt(String key, String text)
            throws NoSuchAlgorithmException, NoSuchPaddingException,
            InvalidKeyException, IllegalBlockSizeException,
            BadPaddingException, UnsupportedEncodingException {
        SecretKeySpec sksSpec = new SecretKeySpec(key.getBytes(ENCODING),
                ALGORITHM_KEY);
        Cipher cipher = Cipher.getInstance(ALGORITHM_CRYPT);
        cipher.init(Cipher.ENCRYPT_MODE, sksSpec);
        byte[] encrypted = cipher.doFinal(text.getBytes(ENCODING));
        return encrypted;
    }

    /**
     * 複合化する
     *
     * @param key
     * @param encrypted
     * @return
     * @throws NoSuchAlgorithmException
     * @throws NoSuchPaddingException
     * @throws InvalidKeyException
     * @throws IllegalBlockSizeException
     * @throws BadPaddingException
     * @throws UnsupportedEncodingException
     */
    public static String decrypt(String key, byte[] encrypted)
            throws NoSuchAlgorithmException, NoSuchPaddingException,
            InvalidKeyException, IllegalBlockSizeException,
            BadPaddingException, UnsupportedEncodingException {
        SecretKeySpec sksSpec = new SecretKeySpec(key.getBytes(ENCODING),
                ALGORITHM_KEY);
        Cipher cipher = Cipher.getInstance(ALGORITHM_CRYPT);
        cipher.init(Cipher.DECRYPT_MODE, sksSpec);
        byte[] decrypted = cipher.doFinal(encrypted);
        return new String(decrypted);
    }

    /**
     * パスワードを暗号化する
     *
     * @param password
     * @return
     * @throws InvalidKeyException
     * @throws NoSuchAlgorithmException
     * @throws NoSuchPaddingException
     * @throws IllegalBlockSizeException
     * @throws BadPaddingException
     * @throws UnsupportedEncodingException
     */
    public static String encryptPassword(String password)
            throws InvalidKeyException, NoSuchAlgorithmException,
            NoSuchPaddingException, IllegalBlockSizeException,
            BadPaddingException, UnsupportedEncodingException {
        return toHexString(encrypt(KEY_PASSWORD, password));
    }

    /**
     * パスワードを複合化する
     *
     * @param encryptedPassword
     * @return
     * @throws InvalidKeyException
     * @throws NoSuchAlgorithmException
     * @throws NoSuchPaddingException
     * @throws IllegalBlockSizeException
     * @throws BadPaddingException
     * @throws UnsupportedEncodingException
     */
    public static String decryptPassword(String encryptedPassword)
            throws InvalidKeyException, NoSuchAlgorithmException,
            NoSuchPaddingException, IllegalBlockSizeException,
            BadPaddingException, UnsupportedEncodingException {
        return decrypt(KEY_PASSWORD, decode(encryptedPassword));
    }

    /**
     * バイト配列を16進数文字列に変換する。
     *
     * @param b
     * @return
     */
    public static String toHexString(byte[] b) {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < b.length; i++) {
            String hex = Integer.toHexString(b[i] & 0xFF);
            if (hex.length() < 2)
                hex = "0" + hex;
            sb.append(hex);
        }
        return new String(sb);
    }

    /**
     * 16進数文字列をバイト配列に変換する。
     *
     * @param value
     * @return
     */
    public static byte[] decode(String value) {
        byte bytes[] = new byte[value.length() / 2];

        for (int i = 0; i < value.length() / 2; i++) {
            StringBuffer hex = new StringBuffer("0x");
            hex.append(value.charAt(i * 2));
            hex.append(value.charAt(i * 2 + 1));
            bytes[i] = Integer.decode(new String(hex)).byteValue();
        }
        return bytes;

    }
}
